%   GNP.PRG 

% This is a MATLAB program, written by Bruce E. Hansen.
% It replicates Table 5 and Figures 3 and 4 from my paper
% "The Grid Bootstrap and the Autoregressive Model".

% If you have questions, please contact me at

% Bruce E. Hansen
% Department of Economics
% Social Science Building
% University of Wisconsin
% Madison, WI 53706-1393
% bhansen@ssc.wisc.edu

function gnp()
global graphin;
global out;
load np_index.txt;
npp=['cpi_____';
     'employmt';
     'gnpdefl_';
     'nomgnp__';
     'interest';
     'indprod_';
     'gnpperca';
     'realgnp_';
     'wages___';
     'realwag_';
     's&p500__';
     'unemploy';
     'velocity'];
np=np_index;
load nplosser.txt;
dat=nplosser;
clear np_index;clear nplosser;

orig=2; % set to 1 for original data, set to 2 for extended data %
t=2;
grph=1; % set to 1 to display graphs %  
grid=200;
bq=3000;
c=.9;

out=fopen('gnp.txt','wt');

i=7;
grpahin=7;
d=[0,np(i,:)];
if orig==1;
    y=dat(d(2):d(3)-18);
    name=strcat('Nelson-Plosser ',num2str(npp(i)));
    if i==4
        y=y(21:82);
    end;
else
    y=dat(d(2):d(3));
    name='GNP per Capital: 1869-1988';
end;
ar=d(4);
grid_boot(y,name,t,ar,grid,bq,c,grph);
fclose(out);


function grid_boot(dat,name,t,ar,grid,bq,c,grph);
global out;
global graphin;
cc=[((1-c)/2);((c+1)/2)];
n=length(dat(:,1));
if t==0 yt=dat; end;
if t==1 yt=dat-mean(dat); end;
if t==2 dt=[ones(n,1),(1:n)'];yt=dat-dt*(dat'/dt')'; end;
y0=yt(1:ar);
[y,x,betahat,sehat,e]= est(dat,t,ar);
g_lu=zeros(ar,2);
b_lu=zeros(ar,2);
k=1;
while k<=ar
    bk=betahat(k+t);
    sk=sehat(k+t);
    beta0=(bk-5*sk:10*sk/(grid-1):bk-5*sk+10*sk)';
    ind=0;
    for ii=1:ar+t
        if (1-(ii==k+t))
            if ind==0
                kk=ii;
                ind=1;
            else
                kk=[kk;ii];
            end;
        end;
    end;
    xk=x(:,kk);
    tstat=zeros(bq,grid);
    i=1;
    while i<=grid
        bi=beta0(i);
        yk=y-x(:,k+t)*bi;
        ai=(yk'/xk')';
        if ar>1
            bbi=zeros(ar,1);
            bbi(k)=bi;
            ind=0;
            for ii=1:ar
                if 1-(ii==k)
                    if ind==0
                        setseq=ii;
                        ind=1;
                    else
                        setseq=[setseq;ii];
                    end;
                end;
            end;
            bbi(setseq)=ai(1+t:ar+t-1);
            phi=[bbi(2:ar);0]-[0;bbi(2:ar)];
            phi(1)=phi(1)+bbi(1);
        else
            phi=bi;
        end;
        rphi=phi(length(phi):-1:1);
        rr=roots([(-rphi);1]);
        if min(abs(rr))<=1;
            y00=zeros(ar,1);
        else
            y00=y0;
        end;
        j=1;
        while j<=bq
            et=e(fix(1+(n-ar)*unifrnd(0,1,n,1)));
            datb=y00;
            for iii=(length(y00(:,1))+1):(length(et(:,1)))
                 datbb=et(iii,:);
                 for jj=1:length(y00(:,1))
                      datbb=datbb+phi(jj,:).*datb(iii-jj,:);
                 end;
                 datb=[datb;datbb];
            end;
            [yb,xb,bb,sb,eb]= est(datb,t,ar);
            tstat(j,i)=(bb(t+k)-bi)/sb(t+k);
            j=j+1;
        end;
        i=i+1;
    end;
    qh=kq_reg(tstat,beta0,cc);      
    tk=((bk-beta0)'/sk')';
    [g_l,g_u]=g_ends(beta0,tk,qh);
    g_lu(k,:)=[g_l,g_u];
    tk
    beta0
    g_l
    g_lh=height(tk,beta0,g_l);
    g_uh=height(tk,beta0,g_u);
    b_lh=height(qh(:,2),beta0,bk);
    b_uh=height(qh(:,1),beta0,bk);
    b_l=bk-b_lh*sk;
    b_u=bk-b_uh*sk;
    b_lu(k,:)=[b_l,b_u];
if grph==1 %?????????????????
        z=zeros(grid,1);
        figure;
        hold on;
        % Points %
        plot(bk,0,'o');
        plot(bk,b_uh,'bd');
        plot(bk,b_lh,'bd');
        plot(b_l,b_lh,'rs');
        plot(b_u,b_uh,'rs');
        plot(g_l,g_lh,'+');
        plot(g_u,g_uh,'+');
        % Lines %
        plot([b_l;bk],[b_lh;b_lh],':');
        plot([bk;bk],[b_lh;b_uh],':');
        plot([bk;b_u],[b_uh;b_uh],':');
        % Arrow %
        plot([g_l;g_l],[g_lh;0]);plot(g_l,0,'*');
        plot([g_u;g_u],[g_uh;0]);plot(g_u,0,'*');
        plot([b_u;b_u],[b_uh;0],':');plot(b_u,0,'*');
        plot([b_l;b_l],[b_lh,0],':');plot(b_l,0,'*');
        % Curve %
        title(strcat(name,'  AR(',num2str(ar),')'));
        xlabel(strcat(name,'  AR(',num2str(ar),')'));
        ylabel('Bootstrap Quantiles')
        plot(beta0,tk,beta0,z,beta0,qh(:,2),beta0,qh(:,1));
        if k==1
            xlim([.7  1.05]);
            ylim([-4 4]);
        elseif k==2
            xlim([.1  .7]);
            ylim([-4 4]);
        end;
        saveas(gcf,strcat('gnp',num2str(graphin),num2str(i),'.eps'),'psc2');
    end;
    k=k+1;
end;
xname='Y(t-1)_';
k=2;
while k<=ar
    kname=strcat('DY(t-',num2str(k),')');
    xname=[xname;kname];
    k=k+1;
end;
fprintf(out,'%s\n',name(1,:));
fprintf(out,'Number of Gridpoints:  %u\n',grid);
fprintf(out,'Number of Bootstrap Replications:  %u\n',bq);

fprintf(out,'Confidence Level:  %f\n\n\n',c);

fprintf(out,'Variable     Estimate     S.E.     Grid_L     Grid_U     Per-t_L     Per-t_U\n');
fprintf(out,'----------------------------------------------------------------------------\n');
for i=1:ar
     fprintf(out,'%s       %f   %f   %f   %f   %f   %f\n',xname(i,:),betahat(i+t),sehat(i+t),g_lu(i,1),g_lu(i,2),b_lu(i,1),b_lu(i,2));
end;

fprintf(out,'----------------------------------------------------------------------------\n\n\n');
fprintf(out,'*********************************\n\n\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC EST
% Format: 	
% 	[y,x,betahat,sehat,e]=est(dat,t,ar);
% Inputs:  
%	dat	nx1 data vector
%	t	trend order.  0 for no trend, 1 for constant, 2 for constant and trend
%	ar	autoregressive order
% Outputs:
% 	y	nx1 vector of dependent variable
% 	x	nx(ar+t) matrix of regressors
% 	betahat	 OLS estimates
%	sehat	OLS standard errors
%	e	OLS residuals

function [y,x,betahat,sehat,e]=est(dat,t,ar);
n=length(dat(:,1));
if t==0
    x=dat(ar:n-1);
elseif t==1
    x=[ones(n-ar,1),dat(ar,n-1)];
elseif t==2
    x=[ones(n-ar,1),(1:n-ar)',dat(ar:n-1)];
end;
k=2;
while k<=ar
    x=[x,(dat(ar-k+2:n-k+1)-dat(ar-k+1:n-k))];
    k=k+1;
end;
y=dat(1+ar:n);
xxi=inv(x'*x);
betahat=xxi*(x'*y);
e=y-x*betahat;
sehat=sqrt(diag(xxi)*(e'*e)/(n-ar-t));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC QUANTILE
% Format: 	
% 	q=quantile(x,c);
% Inputs:  
% 	x	nxr data matrix
% 	c	px1 list of quantiles
% Output:
% 	q	rxp matrix of quantiles

function q=quantile(x,c);
[b,r]=size(x);
xx=x;
j=1;
while j<=r
    xx(:,j)=sortrows(x(:,j),1);
    j=j+1;
end;
bc=b*c;
i=floor(bc);
i=i+(i==0);
q=xx(i,:).*((i==bc)*ones(1,length(xx(1,:))))+((xx(i,:)+xx(i+1,:))/2).*((1-(i==bc))*ones(1,length(xx(1,:))));
q=q';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC EPI
% Format: 	
% 	w=epi(a,h,leave);
% Inputs:  
% 	a	nx1 vector of points to evaluate kernel
% 	h	scalar bandwidth
% 	leave	set to 1 to impose leave-one-out kernel estimation
% Output:
% 	w	nxn matrix of kernel weights

function w=epi(a,h,leave);
aa=a*ones(1,length(a))-ones(length(a),1)*a';
u=(aa'/h)';
w=(1-u.^2).*(abs(u)<=1);

if leave==1
    for i=1:length(w(:,1))
        w(i,i)=0;
    end;
end;
w=w./(sum(w')'*ones(1,length(w(1,:))));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC KQ_REG
% Format: 	
% 	qh=kq_reg(y,x,c);
% Inputs:  
% 	y	nxr data matrix
% 	x	rx1 vector of points to evaluate kernel regression
% 	c	px1 list of quantiles
% Output:
% 	qh	rxp matrix of kernel-smoothed quantiles

function qh=kq_reg(y,x,c);
hn=5;
cn=length(c(:,1));
hh=(1.5:hn+0.5)'*(x(2)-x(1));
check=zeros(hn,cn);
q=quantile(y,c);
hi=1;
while hi<=hn
    h=hh(hi);
    if x(2)-x(1)<h
        qh=epi(x,h,1)*q;
        check(hi,:)=sum((q-qh).^2);
    else
        check(hi,:)=sum(q.^2);
    end;
    hi=hi+1;
end;
[temp,hi]=min(check);
h=hh(hi);
clear temp;
qh=zeros(length(x(:,1)),cn);
ci=1;
while ci<=cn
    qh(:,ci)=(epi(x,h(ci),0)*q(:,ci));
    ci=ci+1;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC CROSS
% Format: 	
% 	[g1,g2]=cross(a,x,y);
% Inputs:  
%	a	rx1 list of gridpoints
%	x	rx1 list, first line
%	c	rx1 list, second line
% Outputs:
%	g1	list of upcrossings (second line intersects first from below)
%	g2	list of downcrossings (second line intersects first from above)

function [g1,g2]=cross(a,x,y);
ms=NaN;      
rn=length(a(:,1));
r=(x<y);
d=[0;(r(2:rn)-r(1:rn-1))];
s=(1:rn)';
ind=0;
id=(d==1);
for i=1:length(id)
    if id(i)==1
        if ind==0
            li=s(i);
            ind=1;
        else
            li=[li;s(i)];
        end;
    end;
end;
if 1-(isnan(sum(li)))
    al=a(li-1);
    au=a(li);
    bl=x(li-1)-y(li-1);
    bu=y(li)-x(li);
    g1=(bu.*al+bl.*au)./(bl+bu);
else
    g1=ms;
end;

ind=0;
id=(d==-1);
for i=1:length(id)
    if id(i)==1
        if ind==0
            li=s(i);
            ind=1;
        else
            li=[li;s(i)];
        end;
    end;
end;
if max(id==0) li=NaN; end;
if 1-(isnan(sum(li)))
    al=a(li-1);
    au=a(li);
    bl=y(li-1)-x(li-1);
    bu=x(li)-y(li);
    g2=(bu.*al+bl.*au)./(bl+bu);
else
    g2=ms;
end;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC G_ENDS
% Format: 	
% 	[l,u]=g_ends(a0,ta,grid);
% Inputs:  
% 	a0	rx1 list of gridpoints
%	ta	rx1 test statistic line
%	grid	rx2 lower and upper quantiles
% Output:
% 	l	lower endpoint of confidence interval
% 	u	upper endpoint of confidence interval
function [l,u]=g_ends(a0,ta,grid);
[l1,l2]=cross(a0,ta,grid(:,1));
[u1,u2]=cross(a0,ta,grid(:,2));
is=sortrows([l1;l2;u1;u2],1);
if length(is(1,:))>1
   iis=isnan(sum(is')');
else
   iis=isnan(is);
end;
ind=0;
for i=1:length(iis)
    if iis(i)==0;
        if ind==0
            s=is(i);
            ind=1;
        else
            s=[s;is(i)];
        end;
    end;
end;
l=min(s);
u=max(s);
if l==min(u2) l=min(a0);end;
if l==min(l1) l=min(a0);end;
if u==max(l2) u=max(a0);end;
if u==max(u1) u=max(a0);end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PROC HEIGHT
% Format: 	
% 	qh=height(q,a,l);
% Inputs:  
% 	q	rx1 heights at gridpoints
% 	a	rx1 gridpoints
% 	l	scalar, point to evaluate q
% Output:
% 	qh	scalar, value of q at point l

function qh=height(q,a,l);
[temp,l2]=max((a>l));
l1=l2-1;
q1=q(l1);
q2=q(l2);
w1=a(l2)-l;
w2=l-a(l1);
qh=(w1*q1+w2*q2)/(w1+w2);
